#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 1.0.2.7202 on 2019-05-14.
#  2019, SMART Health IT.


import os
import io
import unittest
import json
from . import observation
from .fhirdate import FHIRDate


class ObservationTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("Observation", js["resourceType"])
        return observation.Observation(js)
    
    def testObservation1(self):
        inst = self.instantiate_from("obs-genetics-example3-mutationlist-4.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation1(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation1(inst2)
    
    def implObservation1(self, inst):
        self.assertEqual(inst.code.coding[0].code, "49874-1")
        self.assertEqual(inst.code.coding[0].display, "ABCB4 gene mutation analysis")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.component[0].code.coding[0].code, "53037-8")
        self.assertEqual(inst.component[0].code.coding[0].display, "Genetic disease sequence variation interpretation")
        self.assertEqual(inst.component[0].code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.component[0].valueCodeableConcept.coding[0].code, "LA6675-8")
        self.assertEqual(inst.component[0].valueCodeableConcept.coding[0].display, "Benign")
        self.assertEqual(inst.component[0].valueCodeableConcept.coding[0].system, "http://www.genenames.org")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/geneticsDNARegionName")
        self.assertEqual(inst.extension[0].valueString, "intron 26")
        self.assertEqual(inst.extension[1].url, "http://hl7.org/fhir/StructureDefinition/geneticsVariationId")
        self.assertEqual(inst.extension[1].valueCodeableConcept.coding[0].code, "31653")
        self.assertEqual(inst.extension[1].valueCodeableConcept.coding[0].display, "c.3487-16T>C")
        self.assertEqual(inst.extension[1].valueCodeableConcept.coding[0].system, "http://www.ncbi.nlm.nih.gov/projects/SNP")
        self.assertEqual(inst.id, "genetics-example3-mutationlist-4")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
    
    def testObservation2(self):
        inst = self.instantiate_from("observation-example-f206-staphylococcus.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation2(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation2(inst2)
    
    def implObservation2(self, inst):
        self.assertEqual(inst.code.coding[0].code, "104177")
        self.assertEqual(inst.code.coding[0].display, "Blood culture")
        self.assertEqual(inst.code.coding[0].system, "http://acmelabs.org")
        self.assertEqual(inst.code.coding[1].code, "600-7")
        self.assertEqual(inst.code.coding[1].display, "Bacteria identified in Blood by Culture")
        self.assertEqual(inst.code.coding[1].system, "http://loinc.org")
        self.assertEqual(inst.id, "f206")
        self.assertEqual(inst.interpretation.coding[0].code, "POS")
        self.assertEqual(inst.interpretation.coding[0].system, "http://hl7.org/fhir/v2/0078")
        self.assertEqual(inst.issued.date, FHIRDate("2013-03-11T10:28:00+01:00").date)
        self.assertEqual(inst.issued.as_json(), "2013-03-11T10:28:00+01:00")
        self.assertEqual(inst.method.coding[0].code, "104177005")
        self.assertEqual(inst.method.coding[0].display, "Blood culture for bacteria, including anaerobic screen")
        self.assertEqual(inst.method.coding[0].system, "http://snomed.info/sct")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.valueCodeableConcept.coding[0].code, "3092008")
        self.assertEqual(inst.valueCodeableConcept.coding[0].display, "Staphylococcus aureus")
        self.assertEqual(inst.valueCodeableConcept.coding[0].system, "http://snomed.info/sct")
    
    def testObservation3(self):
        inst = self.instantiate_from("observation-example-f202-temperature.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation3(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation3(inst2)
    
    def implObservation3(self, inst):
        self.assertEqual(inst.bodySite.coding[0].code, "74262004")
        self.assertEqual(inst.bodySite.coding[0].display, "Oral cavity")
        self.assertEqual(inst.bodySite.coding[0].system, "http://snomed.info/sct")
        self.assertEqual(inst.code.coding[0].code, "BT")
        self.assertEqual(inst.code.coding[0].display, "Body temperature")
        self.assertEqual(inst.code.coding[0].system, "http://acme.lab")
        self.assertEqual(inst.code.coding[1].code, "8310-5")
        self.assertEqual(inst.code.coding[1].display, "Body temperature")
        self.assertEqual(inst.code.coding[1].system, "http://loinc.org")
        self.assertEqual(inst.code.coding[2].code, "56342008")
        self.assertEqual(inst.code.coding[2].display, "Temperature taking")
        self.assertEqual(inst.code.coding[2].system, "http://snomed.info/sct")
        self.assertEqual(inst.code.text, "Body temperature")
        self.assertEqual(inst.id, "f202")
        self.assertEqual(inst.interpretation.coding[0].code, "H")
        self.assertEqual(inst.interpretation.coding[0].system, "http://hl7.org/fhir/v2/0078")
        self.assertEqual(inst.issued.date, FHIRDate("2013-04-04T13:27:00+01:00").date)
        self.assertEqual(inst.issued.as_json(), "2013-04-04T13:27:00+01:00")
        self.assertEqual(inst.method.coding[0].code, "89003005")
        self.assertEqual(inst.method.coding[0].display, "Oral temperature taking")
        self.assertEqual(inst.method.coding[0].system, "http://snomed.info/sct")
        self.assertEqual(inst.referenceRange[0].low.unit, "degrees C")
        self.assertEqual(inst.referenceRange[0].low.value, 37.5)
        self.assertEqual(inst.status, "entered-in-error")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.valueQuantity.code, "258710007")
        self.assertEqual(inst.valueQuantity.system, "http://snomed.info/sct")
        self.assertEqual(inst.valueQuantity.unit, "degrees C")
        self.assertEqual(inst.valueQuantity.value, 39)
    
    def testObservation4(self):
        inst = self.instantiate_from("observation-example-glasgow-qa.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation4(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation4(inst2)
    
    def implObservation4(self, inst):
        self.assertEqual(inst.code.coding[0].code, "9269-2")
        self.assertEqual(inst.code.coding[0].display, "Glasgow coma score total")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.code.text, "Glasgow Coma Scale , (GCS)")
        self.assertEqual(inst.effectiveDateTime.date, FHIRDate("2014-12-11T04:44:16Z").date)
        self.assertEqual(inst.effectiveDateTime.as_json(), "2014-12-11T04:44:16Z")
        self.assertEqual(inst.id, "gcs-qa")
        self.assertEqual(inst.referenceRange[0].high.code, "{score}")
        self.assertEqual(inst.referenceRange[0].high.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[0].high.value, 8)
        self.assertEqual(inst.referenceRange[0].meaning.text, "Severe TBI")
        self.assertEqual(inst.referenceRange[1].high.code, "{score}")
        self.assertEqual(inst.referenceRange[1].high.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[1].high.value, 12)
        self.assertEqual(inst.referenceRange[1].low.code, "{score}")
        self.assertEqual(inst.referenceRange[1].low.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[1].low.value, 9)
        self.assertEqual(inst.referenceRange[1].meaning.text, "Moderate TBI")
        self.assertEqual(inst.referenceRange[2].low.code, "{score}")
        self.assertEqual(inst.referenceRange[2].low.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[2].low.value, 13)
        self.assertEqual(inst.referenceRange[2].meaning.text, "Mild TBI")
        self.assertEqual(inst.related[0].type, "derived-from")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.valueQuantity.code, "{score}")
        self.assertEqual(inst.valueQuantity.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.valueQuantity.value, 13)
    
    def testObservation5(self):
        inst = self.instantiate_from("obs-genetics-example3-mutationlist-1.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation5(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation5(inst2)
    
    def implObservation5(self, inst):
        self.assertEqual(inst.code.coding[0].code, "49874-1")
        self.assertEqual(inst.code.coding[0].display, "ABCB4 gene mutation analysis")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.component[0].code.coding[0].code, "53037-8")
        self.assertEqual(inst.component[0].code.coding[0].display, "Genetic disease sequence variation interpretation")
        self.assertEqual(inst.component[0].code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.component[0].valueCodeableConcept.coding[0].code, "LA6682-4")
        self.assertEqual(inst.component[0].valueCodeableConcept.coding[0].display, "Unknown significance")
        self.assertEqual(inst.component[0].valueCodeableConcept.coding[0].system, "http://www.genenames.org")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/geneticsDNASequenceVariation")
        self.assertEqual(inst.extension[0].valueString, "c.2708T>C")
        self.assertEqual(inst.extension[1].url, "http://hl7.org/fhir/StructureDefinition/geneticsDNARegionName")
        self.assertEqual(inst.extension[1].valueString, "Exon 23")
        self.assertEqual(inst.extension[2].url, "http://hl7.org/fhir/StructureDefinition/geneticsAminoAcidChange")
        self.assertEqual(inst.extension[2].valueString, "p.R969H")
        self.assertEqual(inst.id, "genetics-example3-mutationlist-1")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
    
    def testObservation6(self):
        inst = self.instantiate_from("observation-example-bloodpressure.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation6(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation6(inst2)
    
    def implObservation6(self, inst):
        self.assertEqual(inst.bodySite.coding[0].code, "368209003")
        self.assertEqual(inst.bodySite.coding[0].display, "Right arm")
        self.assertEqual(inst.bodySite.coding[0].system, "http://snomed.info/sct")
        self.assertEqual(inst.code.coding[0].code, "55284-4")
        self.assertEqual(inst.code.coding[0].display, "Blood pressure systolic & diastolic")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.component[0].code.coding[0].code, "8480-6")
        self.assertEqual(inst.component[0].code.coding[0].display, "Systolic blood pressure")
        self.assertEqual(inst.component[0].code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.component[0].code.coding[1].code, "271649006")
        self.assertEqual(inst.component[0].code.coding[1].display, "Systolic blood pressure")
        self.assertEqual(inst.component[0].code.coding[1].system, "http://snomed.info/sct")
        self.assertEqual(inst.component[0].code.coding[2].code, "bp-s")
        self.assertEqual(inst.component[0].code.coding[2].display, "Systolic Blood pressure")
        self.assertEqual(inst.component[0].code.coding[2].system, "http://acme.org/devices/clinical-codes")
        self.assertEqual(inst.component[0].valueQuantity.unit, "mm[Hg]")
        self.assertEqual(inst.component[0].valueQuantity.value, 107)
        self.assertEqual(inst.component[1].code.coding[0].code, "8462-4")
        self.assertEqual(inst.component[1].code.coding[0].display, "Diastolic blood pressure")
        self.assertEqual(inst.component[1].code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.component[1].valueQuantity.unit, "mm[Hg]")
        self.assertEqual(inst.component[1].valueQuantity.value, 60)
        self.assertEqual(inst.effectiveDateTime.date, FHIRDate("2012-09-17").date)
        self.assertEqual(inst.effectiveDateTime.as_json(), "2012-09-17")
        self.assertEqual(inst.id, "blood-pressure")
        self.assertEqual(inst.identifier[0].system, "urn:ietf:rfc:3986")
        self.assertEqual(inst.identifier[0].value, "urn:uuid:187e0c12-8dd2-67e2-99b2-bf273c878281")
        self.assertEqual(inst.interpretation.coding[0].code, "L")
        self.assertEqual(inst.interpretation.coding[0].display, "Below low normal")
        self.assertEqual(inst.interpretation.coding[0].system, "http://hl7.org/fhir/v2/0078")
        self.assertEqual(inst.interpretation.text, "low")
        self.assertEqual(inst.meta.lastUpdated.date, FHIRDate("2014-01-30T22:35:23+11:00").date)
        self.assertEqual(inst.meta.lastUpdated.as_json(), "2014-01-30T22:35:23+11:00")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
    
    def testObservation7(self):
        inst = self.instantiate_from("obs-genetics-example3-mutationlist-3.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation7(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation7(inst2)
    
    def implObservation7(self, inst):
        self.assertEqual(inst.code.coding[0].code, "49874-1")
        self.assertEqual(inst.code.coding[0].display, "ABCB4 gene mutation analysis")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.component[0].code.coding[0].code, "53037-8")
        self.assertEqual(inst.component[0].code.coding[0].display, "Genetic disease sequence variation interpretation")
        self.assertEqual(inst.component[0].code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.component[0].valueCodeableConcept.coding[0].code, "LA6675-8")
        self.assertEqual(inst.component[0].valueCodeableConcept.coding[0].display, "Benign")
        self.assertEqual(inst.component[0].valueCodeableConcept.coding[0].system, "http://www.genenames.org")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/geneticsDNARegionName")
        self.assertEqual(inst.extension[0].valueString, "intron 16")
        self.assertEqual(inst.extension[1].url, "http://hl7.org/fhir/StructureDefinition/geneticsVariationId")
        self.assertEqual(inst.extension[1].valueCodeableConcept.coding[0].code, "31668")
        self.assertEqual(inst.extension[1].valueCodeableConcept.coding[0].display, "c.2211+16C>T")
        self.assertEqual(inst.extension[1].valueCodeableConcept.coding[0].system, "http://www.ncbi.nlm.nih.gov/projects/SNP")
        self.assertEqual(inst.id, "genetics-example3-mutationlist-3")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
    
    def testObservation8(self):
        inst = self.instantiate_from("observation-example-f005-hemoglobin.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation8(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation8(inst2)
    
    def implObservation8(self, inst):
        self.assertEqual(inst.code.coding[0].code, "718-7")
        self.assertEqual(inst.code.coding[0].display, "Hemoglobin [Mass/volume] in Blood")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.effectivePeriod.end.date, FHIRDate("2013-04-05T10:30:10+01:00").date)
        self.assertEqual(inst.effectivePeriod.end.as_json(), "2013-04-05T10:30:10+01:00")
        self.assertEqual(inst.effectivePeriod.start.date, FHIRDate("2013-04-02T10:30:10+01:00").date)
        self.assertEqual(inst.effectivePeriod.start.as_json(), "2013-04-02T10:30:10+01:00")
        self.assertEqual(inst.id, "f005")
        self.assertEqual(inst.identifier[0].system, "http://www.bmc.nl/zorgportal/identifiers/observations")
        self.assertEqual(inst.identifier[0].use, "official")
        self.assertEqual(inst.identifier[0].value, "6327")
        self.assertEqual(inst.interpretation.coding[0].code, "L")
        self.assertEqual(inst.interpretation.coding[0].display, "Below low normal")
        self.assertEqual(inst.interpretation.coding[0].system, "http://hl7.org/fhir/v2/0078")
        self.assertEqual(inst.issued.date, FHIRDate("2013-04-03T15:30:10+01:00").date)
        self.assertEqual(inst.issued.as_json(), "2013-04-03T15:30:10+01:00")
        self.assertEqual(inst.referenceRange[0].high.code, "g/dL")
        self.assertEqual(inst.referenceRange[0].high.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[0].high.unit, "g/dl")
        self.assertEqual(inst.referenceRange[0].high.value, 10)
        self.assertEqual(inst.referenceRange[0].low.code, "g/dL")
        self.assertEqual(inst.referenceRange[0].low.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[0].low.unit, "g/dl")
        self.assertEqual(inst.referenceRange[0].low.value, 7.5)
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.valueQuantity.code, "g/dL")
        self.assertEqual(inst.valueQuantity.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.valueQuantity.unit, "g/dl")
        self.assertEqual(inst.valueQuantity.value, 7.2)
    
    def testObservation9(self):
        inst = self.instantiate_from("observation-example-glasgow.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation9(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation9(inst2)
    
    def implObservation9(self, inst):
        self.assertEqual(inst.code.coding[0].code, "9269-2")
        self.assertEqual(inst.code.coding[0].display, "Glasgow coma score total")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.code.text, "Glasgow Coma Scale , (GCS)")
        self.assertEqual(inst.contained[0].id, "motor")
        self.assertEqual(inst.contained[1].id, "verbal")
        self.assertEqual(inst.contained[2].id, "eyes")
        self.assertEqual(inst.effectiveDateTime.date, FHIRDate("2014-12-11T04:44:16Z").date)
        self.assertEqual(inst.effectiveDateTime.as_json(), "2014-12-11T04:44:16Z")
        self.assertEqual(inst.id, "glasgow")
        self.assertEqual(inst.referenceRange[0].high.code, "{score}")
        self.assertEqual(inst.referenceRange[0].high.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[0].high.value, 8)
        self.assertEqual(inst.referenceRange[0].meaning.text, "Severe TBI")
        self.assertEqual(inst.referenceRange[1].high.code, "{score}")
        self.assertEqual(inst.referenceRange[1].high.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[1].high.value, 12)
        self.assertEqual(inst.referenceRange[1].low.code, "{score}")
        self.assertEqual(inst.referenceRange[1].low.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[1].low.value, 9)
        self.assertEqual(inst.referenceRange[1].meaning.text, "Moderate TBI")
        self.assertEqual(inst.referenceRange[2].low.code, "{score}")
        self.assertEqual(inst.referenceRange[2].low.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[2].low.value, 13)
        self.assertEqual(inst.referenceRange[2].meaning.text, "Mild TBI")
        self.assertEqual(inst.related[0].type, "derived-from")
        self.assertEqual(inst.related[1].type, "derived-from")
        self.assertEqual(inst.related[2].type, "derived-from")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.valueQuantity.code, "{score}")
        self.assertEqual(inst.valueQuantity.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.valueQuantity.value, 13)
    
    def testObservation10(self):
        inst = self.instantiate_from("observation-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation10(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation10(inst2)
    
    def implObservation10(self, inst):
        self.assertEqual(inst.category.coding[0].code, "vital-signs")
        self.assertEqual(inst.category.coding[0].display, "Vital Signs")
        self.assertEqual(inst.category.coding[0].system, "http://hl7.org/fhir/observation-category")
        self.assertEqual(inst.code.coding[0].code, "3141-9")
        self.assertEqual(inst.code.coding[0].display, "Weight Measured")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.code.coding[1].code, "27113001")
        self.assertEqual(inst.code.coding[1].display, "Body weight")
        self.assertEqual(inst.code.coding[1].system, "http://snomed.info/sct")
        self.assertEqual(inst.code.coding[2].code, "body-weight")
        self.assertEqual(inst.code.coding[2].display, "Body Weight")
        self.assertEqual(inst.code.coding[2].system, "http://acme.org/devices/clinical-codes")
        self.assertEqual(inst.id, "example")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.valueQuantity.code, "[lb_av]")
        self.assertEqual(inst.valueQuantity.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.valueQuantity.unit, "lbs")
        self.assertEqual(inst.valueQuantity.value, 185)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/Consent) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class Consent(domainresource.DomainResource):
    """ A healthcare consumer's  choices to permit or deny recipients or roles to
    perform actions for specific purposes and periods of time.

    A record of a healthcare consumer’s  choices, which permits or denies
    identified recipient(s) or recipient role(s) to perform one or more actions
    within a given policy context, for specific purposes and periods of time.
    """

    resource_type = "Consent"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.category = None
        """ Classification of the consent statement - for indexing/retrieval.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.dateTime = None
        """ When this Consent was created or indexed.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.identifier = None
        """ Identifier for this record (external references).
        List of `Identifier` items (represented as `dict` in JSON). """

        self.organization = None
        """ Custodian of the consent.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.patient = None
        """ Who the consent applies to.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.performer = None
        """ Who is agreeing to the policy and rules.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.policy = None
        """ Policies covered by this consent.
        List of `ConsentPolicy` items (represented as `dict` in JSON). """

        self.policyRule = None
        """ Regulation that this consents to.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.provision = None
        """ Constraints to the base Consent.policyRule.
        Type `ConsentProvision` (represented as `dict` in JSON). """

        self.scope = None
        """ Which of the four areas this resource covers (extensible).
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.sourceAttachment = None
        """ Source from which this consent is taken.
        Type `Attachment` (represented as `dict` in JSON). """

        self.sourceReference = None
        """ Source from which this consent is taken.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.status = None
        """ draft | proposed | active | rejected | inactive | entered-in-error.
        Type `str`. """

        self.verification = None
        """ Consent Verified by patient or family.
        List of `ConsentVerification` items (represented as `dict` in JSON). """

        super(Consent, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(Consent, self).elementProperties()
        js.extend([
            ("category", "category", codeableconcept.CodeableConcept, "CodeableConcept", True, None, True),
            ("dateTime", "dateTime", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", True, None, False),
            ("organization", "organization", fhirreference.FHIRReference, "Reference", True, None, False),
            ("patient", "patient", fhirreference.FHIRReference, "Reference", False, None, False),
            ("performer", "performer", fhirreference.FHIRReference, "Reference", True, None, False),
            ("policy", "policy", ConsentPolicy, "ConsentPolicy", True, None, False),
            ("policyRule", "policyRule", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("provision", "provision", ConsentProvision, "ConsentProvision", False, None, False),
            ("scope", "scope", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
            ("sourceAttachment", "sourceAttachment", attachment.Attachment, "Attachment", False, "source", False),
            ("sourceReference", "sourceReference", fhirreference.FHIRReference, "Reference", False, "source", False),
            ("status", "status", str, "code", False, None, True),
            ("verification", "verification", ConsentVerification, "ConsentVerification", True, None, False),
        ])
        return js


from . import backboneelement

class ConsentPolicy(backboneelement.BackboneElement):
    """ Policies covered by this consent.

    The references to the policies that are included in this consent scope.
    Policies may be organizational, but are often defined jurisdictionally, or
    in law.
    """

    resource_type = "ConsentPolicy"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.authority = None
        """ Enforcement source for policy.
        Type `str`. """

        self.uri = None
        """ Specific policy covered by this consent.
        Type `str`. """

        super(ConsentPolicy, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ConsentPolicy, self).elementProperties()
        js.extend([
            ("authority", "authority", str, "uri", False, None, False),
            ("uri", "uri", str, "uri", False, None, False),
        ])
        return js


class ConsentProvision(backboneelement.BackboneElement):
    """ Constraints to the base Consent.policyRule.

    An exception to the base policy of this consent. An exception can be an
    addition or removal of access permissions.
    """

    resource_type = "ConsentProvision"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.action = None
        """ Actions controlled by this rule.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.actor = None
        """ Who|what controlled by this rule (or group, by role).
        List of `ConsentProvisionActor` items (represented as `dict` in JSON). """

        self.class_fhir = None
        """ e.g. Resource Type, Profile, CDA, etc..
        List of `Coding` items (represented as `dict` in JSON). """

        self.code = None
        """ e.g. LOINC or SNOMED CT code, etc. in the content.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.data = None
        """ Data controlled by this rule.
        List of `ConsentProvisionData` items (represented as `dict` in JSON). """

        self.dataPeriod = None
        """ Timeframe for data controlled by this rule.
        Type `Period` (represented as `dict` in JSON). """

        self.period = None
        """ Timeframe for this rule.
        Type `Period` (represented as `dict` in JSON). """

        self.provision = None
        """ Nested Exception Rules.
        List of `ConsentProvision` items (represented as `dict` in JSON). """

        self.purpose = None
        """ Context of activities covered by this rule.
        List of `Coding` items (represented as `dict` in JSON). """

        self.securityLabel = None
        """ Security Labels that define affected resources.
        List of `Coding` items (represented as `dict` in JSON). """

        self.type = None
        """ deny | permit.
        Type `str`. """

        super(ConsentProvision, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ConsentProvision, self).elementProperties()
        js.extend([
            ("action", "action", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("actor", "actor", ConsentProvisionActor, "ConsentProvisionActor", True, None, False),
            ("class_fhir", "class", coding.Coding, "Coding", True, None, False),
            ("code", "code", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("data", "data", ConsentProvisionData, "ConsentProvisionData", True, None, False),
            ("dataPeriod", "dataPeriod", period.Period, "Period", False, None, False),
            ("period", "period", period.Period, "Period", False, None, False),
            ("provision", "provision", ConsentProvision, "ConsentProvision", True, None, False),
            ("purpose", "purpose", coding.Coding, "Coding", True, None, False),
            ("securityLabel", "securityLabel", coding.Coding, "Coding", True, None, False),
            ("type", "type", str, "code", False, None, False),
        ])
        return js


class ConsentProvisionActor(backboneelement.BackboneElement):
    """ Who|what controlled by this rule (or group, by role).

    Who or what is controlled by this rule. Use group to identify a set of
    actors by some property they share (e.g. 'admitting officers').
    """

    resource_type = "ConsentProvisionActor"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.reference = None
        """ Resource for the actor (or group, by role).
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.role = None
        """ How the actor is involved.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(ConsentProvisionActor, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ConsentProvisionActor, self).elementProperties()
        js.extend([
            ("reference", "reference", fhirreference.FHIRReference, "Reference", False, None, True),
            ("role", "role", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
        ])
        return js


class ConsentProvisionData(backboneelement.BackboneElement):
    """ Data controlled by this rule.

    The resources controlled by this rule if specific resources are referenced.
    """

    resource_type = "ConsentProvisionData"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.meaning = None
        """ instance | related | dependents | authoredby.
        Type `str`. """

        self.reference = None
        """ The actual data reference.
        Type `FHIRReference` (represented as `dict` in JSON). """

        super(ConsentProvisionData, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ConsentProvisionData, self).elementProperties()
        js.extend([
            ("meaning", "meaning", str, "code", False, None, True),
            ("reference", "reference", fhirreference.FHIRReference, "Reference", False, None, True),
        ])
        return js


class ConsentVerification(backboneelement.BackboneElement):
    """ Consent Verified by patient or family.

    Whether a treatment instruction (e.g. artificial respiration yes or no) was
    verified with the patient, his/her family or another authorized person.
    """

    resource_type = "ConsentVerification"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.verificationDate = None
        """ When consent verified.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.verified = None
        """ Has been verified.
        Type `bool`. """

        self.verifiedWith = None
        """ Person who verified.
        Type `FHIRReference` (represented as `dict` in JSON). """

        super(ConsentVerification, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ConsentVerification, self).elementProperties()
        js.extend([
            ("verificationDate", "verificationDate", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("verified", "verified", bool, "boolean", False, None, True),
            ("verifiedWith", "verifiedWith", fhirreference.FHIRReference, "Reference", False, None, False),
        ])
        return js


import sys
try:
    from . import attachment
except ImportError:
    attachment = sys.modules[__package__ + '.attachment']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import coding
except ImportError:
    coding = sys.modules[__package__ + '.coding']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 1.0.2.7202 on 2019-05-14.
#  2019, SMART Health IT.


import os
import io
import unittest
import json
from . import procedurerequest
from .fhirdate import FHIRDate


class ProcedureRequestTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("ProcedureRequest", js["resourceType"])
        return procedurerequest.ProcedureRequest(js)
    
    def testProcedureRequest1(self):
        inst = self.instantiate_from("procedurerequest-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a ProcedureRequest instance")
        self.implProcedureRequest1(inst)
        
        js = inst.as_json()
        self.assertEqual("ProcedureRequest", js["resourceType"])
        inst2 = procedurerequest.ProcedureRequest(js)
        self.implProcedureRequest1(inst2)
    
    def implProcedureRequest1(self, inst):
        self.assertEqual(inst.code.coding[0].code, "323418000")
        self.assertEqual(inst.code.coding[0].display, "Fix me up")
        self.assertEqual(inst.code.coding[0].system, "http://snomed.info/sct")
        self.assertEqual(inst.id, "example")
        self.assertEqual(inst.text.div, "<div>To be added</div>")
        self.assertEqual(inst.text.status, "generated")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b on 2019-05-13.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json

from .fixtures import force_bytes
from .. import examplescenario
from ..fhirdate import FHIRDate


@pytest.mark.usefixtures("base_settings")
class ExampleScenarioTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("ExampleScenario", js["resourceType"])
        return examplescenario.ExampleScenario(js)
    
    def testExampleScenario1(self):
        inst = self.instantiate_from("examplescenario-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a ExampleScenario instance")
        self.implExampleScenario1(inst)
        
        js = inst.as_json()
        self.assertEqual("ExampleScenario", js["resourceType"])
        inst2 = examplescenario.ExampleScenario(js)
        self.implExampleScenario1(inst2)
    
    def implExampleScenario1(self, inst):
        self.assertEqual(force_bytes(inst.actor[0].actorId), force_bytes("Nurse"))
        self.assertEqual(force_bytes(inst.actor[0].description), force_bytes("The Nurse"))
        self.assertEqual(force_bytes(inst.actor[0].name), force_bytes("Nurse"))
        self.assertEqual(force_bytes(inst.actor[0].type), force_bytes("person"))
        self.assertEqual(force_bytes(inst.actor[1].actorId), force_bytes("MAP"))
        self.assertEqual(force_bytes(inst.actor[1].description), force_bytes("The entity that receives the Administration Requests to show the nurse to perform them"))
        self.assertEqual(force_bytes(inst.actor[1].name), force_bytes("Nurse's Tablet"))
        self.assertEqual(force_bytes(inst.actor[1].type), force_bytes("entity"))
        self.assertEqual(force_bytes(inst.actor[2].actorId), force_bytes("OP"))
        self.assertEqual(force_bytes(inst.actor[2].description), force_bytes("The Medication Administration Order Placer"))
        self.assertEqual(force_bytes(inst.actor[2].name), force_bytes("MAR / Scheduler"))
        self.assertEqual(force_bytes(inst.actor[2].type), force_bytes("entity"))
        self.assertEqual(force_bytes(inst.actor[3].actorId), force_bytes("MAC"))
        self.assertEqual(force_bytes(inst.actor[3].description), force_bytes("The entity that receives the Medication Administration reports"))
        self.assertEqual(force_bytes(inst.actor[3].name), force_bytes("MAR / EHR"))
        self.assertEqual(force_bytes(inst.actor[3].type), force_bytes("entity"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example"))
        self.assertEqual(force_bytes(inst.instance[0].description), force_bytes("The initial prescription which describes \"medication X, 3 times per day\" - the exact scheduling is not   in the initial prescription (it is left for the care teams to decide on the schedule)."))
        self.assertEqual(force_bytes(inst.instance[0].name), force_bytes("Initial Prescription"))
        self.assertEqual(force_bytes(inst.instance[0].resourceId), force_bytes("iherx001"))
        self.assertEqual(force_bytes(inst.instance[1].description), force_bytes("The administration request for day 1, morning"))
        self.assertEqual(force_bytes(inst.instance[1].name), force_bytes("Request for day 1, morning"))
        self.assertEqual(force_bytes(inst.instance[1].resourceId), force_bytes("iherx001.001"))
        self.assertEqual(force_bytes(inst.instance[2].description), force_bytes("The administration request for day 1, lunch"))
        self.assertEqual(force_bytes(inst.instance[2].name), force_bytes("Request for day 1, lunch"))
        self.assertEqual(force_bytes(inst.instance[2].resourceId), force_bytes("iherx001.002"))
        self.assertEqual(force_bytes(inst.instance[3].description), force_bytes("The administration request for day 1, evening"))
        self.assertEqual(force_bytes(inst.instance[3].name), force_bytes("Request for day 1, evening"))
        self.assertEqual(force_bytes(inst.instance[3].resourceId), force_bytes("iherx001.003"))
        self.assertEqual(force_bytes(inst.instance[4].description), force_bytes("The administration request for day 2, morning"))
        self.assertEqual(force_bytes(inst.instance[4].name), force_bytes("Request for day 2, morning"))
        self.assertEqual(force_bytes(inst.instance[4].resourceId), force_bytes("iherx001.004"))
        self.assertEqual(force_bytes(inst.instance[5].description), force_bytes("The administration request for day 2, lunch"))
        self.assertEqual(force_bytes(inst.instance[5].name), force_bytes("Request for day 2, lunch"))
        self.assertEqual(force_bytes(inst.instance[5].resourceId), force_bytes("iherx001.005"))
        self.assertEqual(force_bytes(inst.instance[6].description), force_bytes("The administration request for day 2, evening"))
        self.assertEqual(force_bytes(inst.instance[6].name), force_bytes("Request for day 2, evening"))
        self.assertEqual(force_bytes(inst.instance[6].resourceId), force_bytes("iherx001.006"))
        self.assertEqual(force_bytes(inst.instance[7].description), force_bytes("Administration report for day 1, morning: Taken"))
        self.assertEqual(force_bytes(inst.instance[7].name), force_bytes("Morning meds - taken"))
        self.assertEqual(force_bytes(inst.instance[7].resourceId), force_bytes("iheadm001a"))
        self.assertEqual(force_bytes(inst.instance[8].description), force_bytes("Administration report for day 1, morning: NOT Taken"))
        self.assertEqual(force_bytes(inst.instance[8].name), force_bytes("Morning meds - not taken"))
        self.assertEqual(force_bytes(inst.instance[8].resourceId), force_bytes("iheadm001b"))
        self.assertEqual(force_bytes(inst.instance[9].containedInstance[0].resourceId), force_bytes("iherx001.001"))
        self.assertEqual(force_bytes(inst.instance[9].containedInstance[1].resourceId), force_bytes("iherx001.002"))
        self.assertEqual(force_bytes(inst.instance[9].containedInstance[2].resourceId), force_bytes("iherx001.003"))
        self.assertEqual(force_bytes(inst.instance[9].containedInstance[3].resourceId), force_bytes("iherx001.004"))
        self.assertEqual(force_bytes(inst.instance[9].containedInstance[4].resourceId), force_bytes("iherx001.005"))
        self.assertEqual(force_bytes(inst.instance[9].containedInstance[5].resourceId), force_bytes("iherx001.006"))
        self.assertEqual(force_bytes(inst.instance[9].description), force_bytes("All the medication Requests for Day 1"))
        self.assertEqual(force_bytes(inst.instance[9].name), force_bytes("Bundle of Medication Requests"))
        self.assertEqual(force_bytes(inst.instance[9].resourceId), force_bytes("iherx001bundle"))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.process[0].postConditions), force_bytes("Medication administration Reports are submitted, EHR is updated."))
        self.assertEqual(force_bytes(inst.process[0].preConditions), force_bytes("Medication administration requests are in the EHR / MAR, scheduled for each individual intake."))
        self.assertEqual(force_bytes(inst.process[0].step[0].operation.initiator), force_bytes("Nurse"))
        self.assertEqual(force_bytes(inst.process[0].step[0].operation.name), force_bytes("1. Get today's schedule"))
        self.assertEqual(force_bytes(inst.process[0].step[0].operation.number), force_bytes("1"))
        self.assertEqual(force_bytes(inst.process[0].step[0].operation.receiver), force_bytes("MAP"))
        self.assertEqual(force_bytes(inst.process[0].step[1].process[0].description), force_bytes("Query for medication administration orders,\\n- For today's shifts\\n- For today's patients"))
        self.assertEqual(force_bytes(inst.process[0].step[1].process[0].step[0].operation.initiator), force_bytes("MAP"))
        self.assertEqual(force_bytes(inst.process[0].step[1].process[0].step[0].operation.name), force_bytes("2.Query for medication administration orders,\\n- For today's shifts\\n- For today's patients"))
        self.assertEqual(force_bytes(inst.process[0].step[1].process[0].step[0].operation.number), force_bytes("2"))
        self.assertEqual(force_bytes(inst.process[0].step[1].process[0].step[0].operation.receiver), force_bytes("OP"))
        self.assertEqual(force_bytes(inst.process[0].step[1].process[0].step[0].operation.request.resourceId), force_bytes("iherxqry"))
        self.assertEqual(force_bytes(inst.process[0].step[1].process[0].step[0].operation.response.resourceId), force_bytes("iherx001bundle"))
        self.assertEqual(force_bytes(inst.process[0].step[1].process[0].title), force_bytes("P1. Query Administration Requests"))
        self.assertTrue(inst.process[0].step[2].pause)
        self.assertEqual(force_bytes(inst.process[0].step[3].operation.initiator), force_bytes("MAP"))
        self.assertEqual(force_bytes(inst.process[0].step[3].operation.name), force_bytes("Notify (alert)"))
        self.assertEqual(force_bytes(inst.process[0].step[3].operation.number), force_bytes("4"))
        self.assertEqual(force_bytes(inst.process[0].step[3].operation.receiver), force_bytes("Nurse"))
        self.assertEqual(force_bytes(inst.process[0].step[4].operation.initiator), force_bytes("Nurse"))
        self.assertEqual(force_bytes(inst.process[0].step[4].operation.name), force_bytes("Read orders"))
        self.assertEqual(force_bytes(inst.process[0].step[4].operation.number), force_bytes("5"))
        self.assertEqual(force_bytes(inst.process[0].step[4].operation.receiver), force_bytes("MAP"))
        self.assertTrue(inst.process[0].step[5].pause)
        self.assertEqual(force_bytes(inst.process[0].step[6].operation.initiator), force_bytes("Nurse"))
        self.assertEqual(force_bytes(inst.process[0].step[6].operation.name), force_bytes("Ask if patient took meds"))
        self.assertEqual(force_bytes(inst.process[0].step[6].operation.number), force_bytes("5"))
        self.assertEqual(force_bytes(inst.process[0].step[6].operation.receiver), force_bytes("Nurse"))
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[0].description), force_bytes("Invoke if patient took medications"))
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[0].step[0].process[0].step[0].operation.initiator), force_bytes("Nurse"))
        self.assertTrue(inst.process[0].step[7].alternative[0].step[0].process[0].step[0].operation.initiatorActive)
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[0].step[0].process[0].step[0].operation.name), force_bytes("Register Meds taken"))
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[0].step[0].process[0].step[0].operation.number), force_bytes("1a"))
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[0].step[0].process[0].step[0].operation.receiver), force_bytes("MAP"))
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[0].step[0].process[0].title), force_bytes("Register Meds taken"))
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[0].title), force_bytes("Patient took drugs"))
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[1].description), force_bytes("No, patient did not take drugs"))
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[1].step[0].process[0].step[0].operation.initiator), force_bytes("Nurse"))
        self.assertTrue(inst.process[0].step[7].alternative[1].step[0].process[0].step[0].operation.initiatorActive)
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[1].step[0].process[0].step[0].operation.name), force_bytes("Register Meds NOT taken"))
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[1].step[0].process[0].step[0].operation.number), force_bytes("1b"))
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[1].step[0].process[0].step[0].operation.receiver), force_bytes("MAP"))
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[1].step[0].process[0].title), force_bytes("Register Meds NOT taken"))
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[1].title), force_bytes("No drugs"))
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[2].description), force_bytes("Unknown whether patient took medications or not"))
        self.assertTrue(inst.process[0].step[7].alternative[2].step[0].pause)
        self.assertEqual(force_bytes(inst.process[0].step[7].alternative[2].title), force_bytes("Not clear"))
        self.assertTrue(inst.process[0].step[8].pause)
        self.assertEqual(force_bytes(inst.process[0].step[9].operation.initiator), force_bytes("Nurse"))
        self.assertEqual(force_bytes(inst.process[0].step[9].operation.name), force_bytes("Administer drug"))
        self.assertEqual(force_bytes(inst.process[0].step[9].operation.number), force_bytes("6"))
        self.assertEqual(force_bytes(inst.process[0].step[9].operation.receiver), force_bytes("Nurse"))
        self.assertEqual(force_bytes(inst.process[0].title), force_bytes("Mobile Medication Administration"))
        self.assertEqual(force_bytes(inst.status), force_bytes("draft"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/ObservationDefinition) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class ObservationDefinition(domainresource.DomainResource):
    """ Definition of an observation.

    Set of definitional characteristics for a kind of observation or
    measurement produced or consumed by an orderable health care service.
    """

    resource_type = "ObservationDefinition"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.abnormalCodedValueSet = None
        """ Value set of abnormal coded values for the observations conforming
        to this ObservationDefinition.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.category = None
        """ Category of observation.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.code = None
        """ Type of observation (code / type).
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.criticalCodedValueSet = None
        """ Value set of critical coded values for the observations conforming
        to this ObservationDefinition.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.identifier = None
        """ Business identifier for this ObservationDefinition instance.
        List of `Identifier` items (represented as `dict` in JSON). """

        self.method = None
        """ Method used to produce the observation.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.multipleResultsAllowed = None
        """ Multiple results allowed.
        Type `bool`. """

        self.normalCodedValueSet = None
        """ Value set of normal coded values for the observations conforming to
        this ObservationDefinition.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.permittedDataType = None
        """ Quantity | CodeableConcept | string | boolean | integer | Range |
        Ratio | SampledData | time | dateTime | Period.
        List of `str` items. """

        self.preferredReportName = None
        """ Preferred report name.
        Type `str`. """

        self.qualifiedInterval = None
        """ Qualified range for continuous and ordinal observation results.
        List of `ObservationDefinitionQualifiedInterval` items (represented as `dict` in JSON). """

        self.quantitativeDetails = None
        """ Characteristics of quantitative results.
        Type `ObservationDefinitionQuantitativeDetails` (represented as `dict` in JSON). """

        self.validCodedValueSet = None
        """ Value set of valid coded values for the observations conforming to
        this ObservationDefinition.
        Type `FHIRReference` (represented as `dict` in JSON). """

        super(ObservationDefinition, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ObservationDefinition, self).elementProperties()
        js.extend([
            ("abnormalCodedValueSet", "abnormalCodedValueSet", fhirreference.FHIRReference, "Reference", False, None, False),
            ("category", "category", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("code", "code", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
            ("criticalCodedValueSet", "criticalCodedValueSet", fhirreference.FHIRReference, "Reference", False, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", True, None, False),
            ("method", "method", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("multipleResultsAllowed", "multipleResultsAllowed", bool, "boolean", False, None, False),
            ("normalCodedValueSet", "normalCodedValueSet", fhirreference.FHIRReference, "Reference", False, None, False),
            ("permittedDataType", "permittedDataType", str, "code", True, None, False),
            ("preferredReportName", "preferredReportName", str, "string", False, None, False),
            ("qualifiedInterval", "qualifiedInterval", ObservationDefinitionQualifiedInterval, "ObservationDefinitionQualifiedInterval", True, None, False),
            ("quantitativeDetails", "quantitativeDetails", ObservationDefinitionQuantitativeDetails, "ObservationDefinitionQuantitativeDetails", False, None, False),
            ("validCodedValueSet", "validCodedValueSet", fhirreference.FHIRReference, "Reference", False, None, False),
        ])
        return js


from . import backboneelement

class ObservationDefinitionQualifiedInterval(backboneelement.BackboneElement):
    """ Qualified range for continuous and ordinal observation results.

    Multiple  ranges of results qualified by different contexts for ordinal or
    continuous observations conforming to this ObservationDefinition.
    """

    resource_type = "ObservationDefinitionQualifiedInterval"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.age = None
        """ Applicable age range, if relevant.
        Type `Range` (represented as `dict` in JSON). """

        self.appliesTo = None
        """ Targetted population of the range.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.category = None
        """ reference | critical | absolute.
        Type `str`. """

        self.condition = None
        """ Condition associated with the reference range.
        Type `str`. """

        self.context = None
        """ Range context qualifier.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.gender = None
        """ male | female | other | unknown.
        Type `str`. """

        self.gestationalAge = None
        """ Applicable gestational age range, if relevant.
        Type `Range` (represented as `dict` in JSON). """

        self.range = None
        """ The interval itself, for continuous or ordinal observations.
        Type `Range` (represented as `dict` in JSON). """

        super(ObservationDefinitionQualifiedInterval, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ObservationDefinitionQualifiedInterval, self).elementProperties()
        js.extend([
            ("age", "age", range.Range, "Range", False, None, False),
            ("appliesTo", "appliesTo", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("category", "category", str, "code", False, None, False),
            ("condition", "condition", str, "string", False, None, False),
            ("context", "context", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("gender", "gender", str, "code", False, None, False),
            ("gestationalAge", "gestationalAge", range.Range, "Range", False, None, False),
            ("range", "range", range.Range, "Range", False, None, False),
        ])
        return js


class ObservationDefinitionQuantitativeDetails(backboneelement.BackboneElement):
    """ Characteristics of quantitative results.

    Characteristics for quantitative results of this observation.
    """

    resource_type = "ObservationDefinitionQuantitativeDetails"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.conversionFactor = None
        """ SI to Customary unit conversion factor.
        Type `float`. """

        self.customaryUnit = None
        """ Customary unit for quantitative results.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.decimalPrecision = None
        """ Decimal precision of observation quantitative results.
        Type `int`. """

        self.unit = None
        """ SI unit for quantitative results.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(ObservationDefinitionQuantitativeDetails, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ObservationDefinitionQuantitativeDetails, self).elementProperties()
        js.extend([
            ("conversionFactor", "conversionFactor", float, "decimal", False, None, False),
            ("customaryUnit", "customaryUnit", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("decimalPrecision", "decimalPrecision", int, "integer", False, None, False),
            ("unit", "unit", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import range
except ImportError:
    range = sys.modules[__package__ + '.range']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/SubstanceReferenceInformation) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class SubstanceReferenceInformation(domainresource.DomainResource):
    """ Todo.
    """

    resource_type = "SubstanceReferenceInformation"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.classification = None
        """ Todo.
        List of `SubstanceReferenceInformationClassification` items (represented as `dict` in JSON). """

        self.comment = None
        """ Todo.
        Type `str`. """

        self.gene = None
        """ Todo.
        List of `SubstanceReferenceInformationGene` items (represented as `dict` in JSON). """

        self.geneElement = None
        """ Todo.
        List of `SubstanceReferenceInformationGeneElement` items (represented as `dict` in JSON). """

        self.target = None
        """ Todo.
        List of `SubstanceReferenceInformationTarget` items (represented as `dict` in JSON). """

        super(SubstanceReferenceInformation, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(SubstanceReferenceInformation, self).elementProperties()
        js.extend([
            ("classification", "classification", SubstanceReferenceInformationClassification, "SubstanceReferenceInformationClassification", True, None, False),
            ("comment", "comment", str, "string", False, None, False),
            ("gene", "gene", SubstanceReferenceInformationGene, "SubstanceReferenceInformationGene", True, None, False),
            ("geneElement", "geneElement", SubstanceReferenceInformationGeneElement, "SubstanceReferenceInformationGeneElement", True, None, False),
            ("target", "target", SubstanceReferenceInformationTarget, "SubstanceReferenceInformationTarget", True, None, False),
        ])
        return js


from . import backboneelement

class SubstanceReferenceInformationClassification(backboneelement.BackboneElement):
    """ Todo.
    """

    resource_type = "SubstanceReferenceInformationClassification"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.classification = None
        """ Todo.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.domain = None
        """ Todo.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.source = None
        """ Todo.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.subtype = None
        """ Todo.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        super(SubstanceReferenceInformationClassification, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(SubstanceReferenceInformationClassification, self).elementProperties()
        js.extend([
            ("classification", "classification", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("domain", "domain", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("source", "source", fhirreference.FHIRReference, "Reference", True, None, False),
            ("subtype", "subtype", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
        ])
        return js


class SubstanceReferenceInformationGene(backboneelement.BackboneElement):
    """ Todo.
    """

    resource_type = "SubstanceReferenceInformationGene"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.gene = None
        """ Todo.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.geneSequenceOrigin = None
        """ Todo.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.source = None
        """ Todo.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        super(SubstanceReferenceInformationGene, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(SubstanceReferenceInformationGene, self).elementProperties()
        js.extend([
            ("gene", "gene", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("geneSequenceOrigin", "geneSequenceOrigin", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("source", "source", fhirreference.FHIRReference, "Reference", True, None, False),
        ])
        return js


class SubstanceReferenceInformationGeneElement(backboneelement.BackboneElement):
    """ Todo.
    """

    resource_type = "SubstanceReferenceInformationGeneElement"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.element = None
        """ Todo.
        Type `Identifier` (represented as `dict` in JSON). """

        self.source = None
        """ Todo.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.type = None
        """ Todo.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(SubstanceReferenceInformationGeneElement, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(SubstanceReferenceInformationGeneElement, self).elementProperties()
        js.extend([
            ("element", "element", identifier.Identifier, "Identifier", False, None, False),
            ("source", "source", fhirreference.FHIRReference, "Reference", True, None, False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
        ])
        return js


class SubstanceReferenceInformationTarget(backboneelement.BackboneElement):
    """ Todo.
    """

    resource_type = "SubstanceReferenceInformationTarget"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.amountQuantity = None
        """ Todo.
        Type `Quantity` (represented as `dict` in JSON). """

        self.amountRange = None
        """ Todo.
        Type `Range` (represented as `dict` in JSON). """

        self.amountString = None
        """ Todo.
        Type `str`. """

        self.amountType = None
        """ Todo.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.interaction = None
        """ Todo.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.organism = None
        """ Todo.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.organismType = None
        """ Todo.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.source = None
        """ Todo.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.target = None
        """ Todo.
        Type `Identifier` (represented as `dict` in JSON). """

        self.type = None
        """ Todo.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(SubstanceReferenceInformationTarget, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(SubstanceReferenceInformationTarget, self).elementProperties()
        js.extend([
            ("amountQuantity", "amountQuantity", quantity.Quantity, "Quantity", False, "amount", False),
            ("amountRange", "amountRange", range.Range, "Range", False, "amount", False),
            ("amountString", "amountString", str, "string", False, "amount", False),
            ("amountType", "amountType", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("interaction", "interaction", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("organism", "organism", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("organismType", "organismType", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("source", "source", fhirreference.FHIRReference, "Reference", True, None, False),
            ("target", "target", identifier.Identifier, "Identifier", False, None, False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import quantity
except ImportError:
    quantity = sys.modules[__package__ + '.quantity']
try:
    from . import range
except ImportError:
    range = sys.modules[__package__ + '.range']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/Device) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class Device(domainresource.DomainResource):
    """ Item used in healthcare.

    A type of a manufactured item that is used in the provision of healthcare
    without being substantially changed through that activity. The device may
    be a medical or non-medical device.
    """

    resource_type = "Device"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.contact = None
        """ Details for human/organization for support.
        List of `ContactPoint` items (represented as `dict` in JSON). """

        self.definition = None
        """ The reference to the definition for the device.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.deviceName = None
        """ The name of the device as given by the manufacturer.
        List of `DeviceDeviceName` items (represented as `dict` in JSON). """

        self.distinctIdentifier = None
        """ The distinct identification string.
        Type `str`. """

        self.expirationDate = None
        """ Date and time of expiry of this device (if applicable).
        Type `FHIRDate` (represented as `str` in JSON). """

        self.identifier = None
        """ Instance identifier.
        List of `Identifier` items (represented as `dict` in JSON). """

        self.location = None
        """ Where the device is found.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.lotNumber = None
        """ Lot number of manufacture.
        Type `str`. """

        self.manufactureDate = None
        """ Date when the device was made.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.manufacturer = None
        """ Name of device manufacturer.
        Type `str`. """

        self.modelNumber = None
        """ The model number for the device.
        Type `str`. """

        self.note = None
        """ Device notes and comments.
        List of `Annotation` items (represented as `dict` in JSON). """

        self.owner = None
        """ Organization responsible for device.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.parent = None
        """ The parent device.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.partNumber = None
        """ The part number of the device.
        Type `str`. """

        self.patient = None
        """ Patient to whom Device is affixed.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.property = None
        """ The actual configuration settings of a device as it actually
        operates, e.g., regulation status, time properties.
        List of `DeviceProperty` items (represented as `dict` in JSON). """

        self.safety = None
        """ Safety Characteristics of Device.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.serialNumber = None
        """ Serial number assigned by the manufacturer.
        Type `str`. """

        self.specialization = None
        """ The capabilities supported on a  device, the standards to which the
        device conforms for a particular purpose, and used for the
        communication.
        List of `DeviceSpecialization` items (represented as `dict` in JSON). """

        self.status = None
        """ active | inactive | entered-in-error | unknown.
        Type `str`. """

        self.statusReason = None
        """ online | paused | standby | offline | not-ready | transduc-discon |
        hw-discon | off.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.type = None
        """ The kind or type of device.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.udiCarrier = None
        """ Unique Device Identifier (UDI) Barcode string.
        List of `DeviceUdiCarrier` items (represented as `dict` in JSON). """

        self.url = None
        """ Network address to contact device.
        Type `str`. """

        self.version = None
        """ The actual design of the device or software version running on the
        device.
        List of `DeviceVersion` items (represented as `dict` in JSON). """

        super(Device, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(Device, self).elementProperties()
        js.extend([
            ("contact", "contact", contactpoint.ContactPoint, "ContactPoint", True, None, False),
            ("definition", "definition", fhirreference.FHIRReference, "Reference", False, None, False),
            ("deviceName", "deviceName", DeviceDeviceName, "DeviceDeviceName", True, None, False),
            ("distinctIdentifier", "distinctIdentifier", str, "string", False, None, False),
            ("expirationDate", "expirationDate", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", True, None, False),
            ("location", "location", fhirreference.FHIRReference, "Reference", False, None, False),
            ("lotNumber", "lotNumber", str, "string", False, None, False),
            ("manufactureDate", "manufactureDate", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("manufacturer", "manufacturer", str, "string", False, None, False),
            ("modelNumber", "modelNumber", str, "string", False, None, False),
            ("note", "note", annotation.Annotation, "Annotation", True, None, False),
            ("owner", "owner", fhirreference.FHIRReference, "Reference", False, None, False),
            ("parent", "parent", fhirreference.FHIRReference, "Reference", False, None, False),
            ("partNumber", "partNumber", str, "string", False, None, False),
            ("patient", "patient", fhirreference.FHIRReference, "Reference", False, None, False),
            ("property", "property", DeviceProperty, "DeviceProperty", True, None, False),
            ("safety", "safety", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("serialNumber", "serialNumber", str, "string", False, None, False),
            ("specialization", "specialization", DeviceSpecialization, "DeviceSpecialization", True, None, False),
            ("status", "status", str, "code", False, None, False),
            ("statusReason", "statusReason", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("udiCarrier", "udiCarrier", DeviceUdiCarrier, "DeviceUdiCarrier", True, None, False),
            ("url", "url", str, "uri", False, None, False),
            ("version", "version", DeviceVersion, "DeviceVersion", True, None, False),
        ])
        return js


from . import backboneelement

class DeviceDeviceName(backboneelement.BackboneElement):
    """ The name of the device as given by the manufacturer.

    This represents the manufacturer's name of the device as provided by the
    device, from a UDI label, or by a person describing the Device.  This
    typically would be used when a person provides the name(s) or when the
    device represents one of the names available from DeviceDefinition.
    """

    resource_type = "DeviceDeviceName"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.name = None
        """ The name of the device.
        Type `str`. """

        self.type = None
        """ udi-label-name | user-friendly-name | patient-reported-name |
        manufacturer-name | model-name | other.
        Type `str`. """

        super(DeviceDeviceName, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(DeviceDeviceName, self).elementProperties()
        js.extend([
            ("name", "name", str, "string", False, None, True),
            ("type", "type", str, "code", False, None, True),
        ])
        return js


class DeviceProperty(backboneelement.BackboneElement):
    """ The actual configuration settings of a device as it actually operates,
    e.g., regulation status, time properties.
    """

    resource_type = "DeviceProperty"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.type = None
        """ Code that specifies the property DeviceDefinitionPropetyCode
        (Extensible).
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.valueCode = None
        """ Property value as a code, e.g., NTP4 (synced to NTP).
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.valueQuantity = None
        """ Property value as a quantity.
        List of `Quantity` items (represented as `dict` in JSON). """

        super(DeviceProperty, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(DeviceProperty, self).elementProperties()
        js.extend([
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
            ("valueCode", "valueCode", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("valueQuantity", "valueQuantity", quantity.Quantity, "Quantity", True, None, False),
        ])
        return js


class DeviceSpecialization(backboneelement.BackboneElement):
    """ The capabilities supported on a  device, the standards to which the device
    conforms for a particular purpose, and used for the communication.
    """

    resource_type = "DeviceSpecialization"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.systemType = None
        """ The standard that is used to operate and communicate.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.version = None
        """ The version of the standard that is used to operate and communicate.
        Type `str`. """

        super(DeviceSpecialization, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(DeviceSpecialization, self).elementProperties()
        js.extend([
            ("systemType", "systemType", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
            ("version", "version", str, "string", False, None, False),
        ])
        return js


class DeviceUdiCarrier(backboneelement.BackboneElement):
    """ Unique Device Identifier (UDI) Barcode string.

    Unique device identifier (UDI) assigned to device label or package.  Note
    that the Device may include multiple udiCarriers as it either may include
    just the udiCarrier for the jurisdiction it is sold, or for multiple
    jurisdictions it could have been sold.
    """

    resource_type = "DeviceUdiCarrier"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.carrierAIDC = None
        """ UDI Machine Readable Barcode String.
        Type `str`. """

        self.carrierHRF = None
        """ UDI Human Readable Barcode String.
        Type `str`. """

        self.deviceIdentifier = None
        """ Mandatory fixed portion of UDI.
        Type `str`. """

        self.entryType = None
        """ barcode | rfid | manual +.
        Type `str`. """

        self.issuer = None
        """ UDI Issuing Organization.
        Type `str`. """

        self.jurisdiction = None
        """ Regional UDI authority.
        Type `str`. """

        super(DeviceUdiCarrier, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(DeviceUdiCarrier, self).elementProperties()
        js.extend([
            ("carrierAIDC", "carrierAIDC", str, "base64Binary", False, None, False),
            ("carrierHRF", "carrierHRF", str, "string", False, None, False),
            ("deviceIdentifier", "deviceIdentifier", str, "string", False, None, False),
            ("entryType", "entryType", str, "code", False, None, False),
            ("issuer", "issuer", str, "uri", False, None, False),
            ("jurisdiction", "jurisdiction", str, "uri", False, None, False),
        ])
        return js


class DeviceVersion(backboneelement.BackboneElement):
    """ The actual design of the device or software version running on the device.
    """

    resource_type = "DeviceVersion"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.component = None
        """ A single component of the device version.
        Type `Identifier` (represented as `dict` in JSON). """

        self.type = None
        """ The type of the device version.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.value = None
        """ The version text.
        Type `str`. """

        super(DeviceVersion, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(DeviceVersion, self).elementProperties()
        js.extend([
            ("component", "component", identifier.Identifier, "Identifier", False, None, False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("value", "value", str, "string", False, None, True),
        ])
        return js


import sys
try:
    from . import annotation
except ImportError:
    annotation = sys.modules[__package__ + '.annotation']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import contactpoint
except ImportError:
    contactpoint = sys.modules[__package__ + '.contactpoint']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import quantity
except ImportError:
    quantity = sys.modules[__package__ + '.quantity']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b on 2019-05-13.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json

from .fixtures import force_bytes
from .. import coverageeligibilityresponse
from ..fhirdate import FHIRDate


@pytest.mark.usefixtures("base_settings")
class CoverageEligibilityResponseTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("CoverageEligibilityResponse", js["resourceType"])
        return coverageeligibilityresponse.CoverageEligibilityResponse(js)
    
    def testCoverageEligibilityResponse1(self):
        inst = self.instantiate_from("coverageeligibilityresponse-example-error.json")
        self.assertIsNotNone(inst, "Must have instantiated a CoverageEligibilityResponse instance")
        self.implCoverageEligibilityResponse1(inst)
        
        js = inst.as_json()
        self.assertEqual("CoverageEligibilityResponse", js["resourceType"])
        inst2 = coverageeligibilityresponse.CoverageEligibilityResponse(js)
        self.implCoverageEligibilityResponse1(inst2)
    
    def implCoverageEligibilityResponse1(self, inst):
        self.assertEqual(inst.created.date, FHIRDate("2014-09-16").date)
        self.assertEqual(inst.created.as_json(), "2014-09-16")
        self.assertEqual(force_bytes(inst.disposition), force_bytes("Eligibiliy request could not be processed, please address errors before submitting."))
        self.assertEqual(force_bytes(inst.error[0].code.coding[0].code), force_bytes("a001"))
        self.assertEqual(force_bytes(inst.error[0].code.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/adjudication-error"))
        self.assertEqual(force_bytes(inst.form.coding[0].code), force_bytes("ELRSP/2017/01"))
        self.assertEqual(force_bytes(inst.form.coding[0].system), force_bytes("http://national.org/form"))
        self.assertEqual(force_bytes(inst.id), force_bytes("E2503"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://www.BenefitsInc.com/fhir/coverageeligibilityresponse"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("8812343"))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("error"))
        self.assertEqual(force_bytes(inst.purpose[0]), force_bytes("validation"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A human-readable rendering of the CoverageEligibilityResponse.</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testCoverageEligibilityResponse2(self):
        inst = self.instantiate_from("coverageeligibilityresponse-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a CoverageEligibilityResponse instance")
        self.implCoverageEligibilityResponse2(inst)
        
        js = inst.as_json()
        self.assertEqual("CoverageEligibilityResponse", js["resourceType"])
        inst2 = coverageeligibilityresponse.CoverageEligibilityResponse(js)
        self.implCoverageEligibilityResponse2(inst2)
    
    def implCoverageEligibilityResponse2(self, inst):
        self.assertEqual(inst.created.date, FHIRDate("2014-08-16").date)
        self.assertEqual(inst.created.as_json(), "2014-08-16")
        self.assertEqual(force_bytes(inst.disposition), force_bytes("Policy is currently in-force."))
        self.assertEqual(force_bytes(inst.id), force_bytes("E2500"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://www.BenefitsInc.com/fhir/coverageeligibilityresponse"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("881234"))
        self.assertTrue(inst.insurance[0].inforce)
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("complete"))
        self.assertEqual(force_bytes(inst.purpose[0]), force_bytes("validation"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A human-readable rendering of the CoverageEligibilityResponse.</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testCoverageEligibilityResponse3(self):
        inst = self.instantiate_from("coverageeligibilityresponse-example-benefits-2.json")
        self.assertIsNotNone(inst, "Must have instantiated a CoverageEligibilityResponse instance")
        self.implCoverageEligibilityResponse3(inst)
        
        js = inst.as_json()
        self.assertEqual("CoverageEligibilityResponse", js["resourceType"])
        inst2 = coverageeligibilityresponse.CoverageEligibilityResponse(js)
        self.implCoverageEligibilityResponse3(inst2)
    
    def implCoverageEligibilityResponse3(self, inst):
        self.assertEqual(force_bytes(inst.contained[0].id), force_bytes("coverage-1"))
        self.assertEqual(inst.created.date, FHIRDate("2014-09-16").date)
        self.assertEqual(inst.created.as_json(), "2014-09-16")
        self.assertEqual(force_bytes(inst.disposition), force_bytes("Policy is currently in-force."))
        self.assertEqual(force_bytes(inst.form.coding[0].code), force_bytes("ELRSP/2017/01"))
        self.assertEqual(force_bytes(inst.form.coding[0].system), force_bytes("http://national.org/form"))
        self.assertEqual(force_bytes(inst.id), force_bytes("E2502"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://www.BenefitsInc.com/fhir/coverageeligibilityresponse"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("8812342"))
        self.assertTrue(inst.insurance[0].inforce)
        self.assertEqual(force_bytes(inst.insurance[0].item[0].benefit[0].allowedMoney.currency), force_bytes("USD"))
        self.assertEqual(inst.insurance[0].item[0].benefit[0].allowedMoney.value, 500000)
        self.assertEqual(force_bytes(inst.insurance[0].item[0].benefit[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].benefit[0].usedMoney.currency), force_bytes("USD"))
        self.assertEqual(inst.insurance[0].item[0].benefit[0].usedMoney.value, 3748.0)
        self.assertEqual(force_bytes(inst.insurance[0].item[0].benefit[1].allowedMoney.currency), force_bytes("USD"))
        self.assertEqual(inst.insurance[0].item[0].benefit[1].allowedMoney.value, 100)
        self.assertEqual(force_bytes(inst.insurance[0].item[0].benefit[1].type.coding[0].code), force_bytes("copay-maximum"))
        self.assertEqual(inst.insurance[0].item[0].benefit[2].allowedUnsignedInt, 20)
        self.assertEqual(force_bytes(inst.insurance[0].item[0].benefit[2].type.coding[0].code), force_bytes("copay-percent"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].category.coding[0].code), force_bytes("30"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].category.coding[0].display), force_bytes("Health Benefit Plan Coverage"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].category.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/ex-benefitcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].network.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].term.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].unit.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].benefit[0].allowedMoney.currency), force_bytes("USD"))
        self.assertEqual(inst.insurance[0].item[1].benefit[0].allowedMoney.value, 15000)
        self.assertEqual(force_bytes(inst.insurance[0].item[1].benefit[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].category.coding[0].code), force_bytes("69"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].category.coding[0].display), force_bytes("Maternity"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].category.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/ex-benefitcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].network.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].term.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].unit.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].benefit[0].allowedMoney.currency), force_bytes("USD"))
        self.assertEqual(inst.insurance[0].item[2].benefit[0].allowedMoney.value, 2000)
        self.assertEqual(force_bytes(inst.insurance[0].item[2].benefit[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].category.coding[0].code), force_bytes("F3"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].category.coding[0].display), force_bytes("Dental Coverage"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].category.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/ex-benefitcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].network.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].term.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].unit.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].category.coding[0].code), force_bytes("F6"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].category.coding[0].display), force_bytes("Vision Coverage"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].category.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/ex-benefitcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].description), force_bytes("Vision products and services such as exams, glasses and contact lenses."))
        self.assertTrue(inst.insurance[0].item[3].excluded)
        self.assertEqual(force_bytes(inst.insurance[0].item[3].name), force_bytes("Vision"))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("complete"))
        self.assertEqual(force_bytes(inst.purpose[0]), force_bytes("validation"))
        self.assertEqual(force_bytes(inst.purpose[1]), force_bytes("benefits"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A human-readable rendering of the CoverageEligibilityResponse.</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testCoverageEligibilityResponse4(self):
        inst = self.instantiate_from("coverageeligibilityresponse-example-benefits.json")
        self.assertIsNotNone(inst, "Must have instantiated a CoverageEligibilityResponse instance")
        self.implCoverageEligibilityResponse4(inst)
        
        js = inst.as_json()
        self.assertEqual("CoverageEligibilityResponse", js["resourceType"])
        inst2 = coverageeligibilityresponse.CoverageEligibilityResponse(js)
        self.implCoverageEligibilityResponse4(inst2)
    
    def implCoverageEligibilityResponse4(self, inst):
        self.assertEqual(inst.created.date, FHIRDate("2014-08-16").date)
        self.assertEqual(inst.created.as_json(), "2014-08-16")
        self.assertEqual(force_bytes(inst.disposition), force_bytes("Policy is currently in-force."))
        self.assertEqual(force_bytes(inst.id), force_bytes("E2501"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://www.BenefitsInc.com/fhir/coverageeligibilityresponse"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("881234"))
        self.assertTrue(inst.insurance[0].inforce)
        self.assertEqual(force_bytes(inst.insurance[0].item[0].benefit[0].allowedMoney.currency), force_bytes("SAR"))
        self.assertEqual(inst.insurance[0].item[0].benefit[0].allowedMoney.value, 500000)
        self.assertEqual(force_bytes(inst.insurance[0].item[0].benefit[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].benefit[1].allowedMoney.currency), force_bytes("SAR"))
        self.assertEqual(inst.insurance[0].item[0].benefit[1].allowedMoney.value, 100)
        self.assertEqual(force_bytes(inst.insurance[0].item[0].benefit[1].type.coding[0].code), force_bytes("copay-maximum"))
        self.assertEqual(inst.insurance[0].item[0].benefit[2].allowedUnsignedInt, 20)
        self.assertEqual(force_bytes(inst.insurance[0].item[0].benefit[2].type.coding[0].code), force_bytes("copay-percent"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].category.coding[0].code), force_bytes("30"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].category.coding[0].display), force_bytes("Health Benefit Plan Coverage"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].category.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/ex-benefitcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].network.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].term.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[0].unit.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].benefit[0].allowedMoney.currency), force_bytes("SAR"))
        self.assertEqual(inst.insurance[0].item[1].benefit[0].allowedMoney.value, 15000)
        self.assertEqual(force_bytes(inst.insurance[0].item[1].benefit[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].category.coding[0].code), force_bytes("69"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].category.coding[0].display), force_bytes("Maternity"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].category.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/ex-benefitcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].network.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].term.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[1].unit.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].benefit[0].allowedMoney.currency), force_bytes("SAR"))
        self.assertEqual(inst.insurance[0].item[2].benefit[0].allowedMoney.value, 2000)
        self.assertEqual(force_bytes(inst.insurance[0].item[2].benefit[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].category.coding[0].code), force_bytes("F3"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].category.coding[0].display), force_bytes("Dental Coverage"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].category.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/ex-benefitcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].network.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].term.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[2].unit.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].benefit[0].allowedMoney.currency), force_bytes("SAR"))
        self.assertEqual(inst.insurance[0].item[3].benefit[0].allowedMoney.value, 400)
        self.assertEqual(force_bytes(inst.insurance[0].item[3].benefit[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].category.coding[0].code), force_bytes("F6"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].category.coding[0].display), force_bytes("Vision Coverage"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].category.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/ex-benefitcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].network.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].term.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[3].unit.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[4].benefit[0].allowedString), force_bytes("shared"))
        self.assertEqual(force_bytes(inst.insurance[0].item[4].benefit[0].type.coding[0].code), force_bytes("room"))
        self.assertEqual(force_bytes(inst.insurance[0].item[4].benefit[1].allowedMoney.currency), force_bytes("SAR"))
        self.assertEqual(inst.insurance[0].item[4].benefit[1].allowedMoney.value, 600)
        self.assertEqual(force_bytes(inst.insurance[0].item[4].benefit[1].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].item[4].category.coding[0].code), force_bytes("49"))
        self.assertEqual(force_bytes(inst.insurance[0].item[4].category.coding[0].display), force_bytes("Hospital Room and Board"))
        self.assertEqual(force_bytes(inst.insurance[0].item[4].category.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/ex-benefitcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].item[4].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].item[4].network.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].item[4].term.coding[0].code), force_bytes("day"))
        self.assertEqual(force_bytes(inst.insurance[0].item[4].term.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].item[4].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].item[4].unit.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/benefit-unit"))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("complete"))
        self.assertEqual(force_bytes(inst.purpose[0]), force_bytes("validation"))
        self.assertEqual(force_bytes(inst.purpose[1]), force_bytes("benefits"))
        self.assertEqual(inst.servicedDate.date, FHIRDate("2014-09-17").date)
        self.assertEqual(inst.servicedDate.as_json(), "2014-09-17")
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A human-readable rendering of the CoverageEligibilityResponse.</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/CoverageEligibilityResponse) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class CoverageEligibilityResponse(domainresource.DomainResource):
    """ CoverageEligibilityResponse resource.

    This resource provides eligibility and plan details from the processing of
    an CoverageEligibilityRequest resource.
    """

    resource_type = "CoverageEligibilityResponse"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.created = None
        """ Response creation date.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.disposition = None
        """ Disposition Message.
        Type `str`. """

        self.error = None
        """ Processing errors.
        List of `CoverageEligibilityResponseError` items (represented as `dict` in JSON). """

        self.form = None
        """ Printed form identifier.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.identifier = None
        """ Business Identifier for coverage eligiblity request.
        List of `Identifier` items (represented as `dict` in JSON). """

        self.insurance = None
        """ Patient insurance information.
        List of `CoverageEligibilityResponseInsurance` items (represented as `dict` in JSON). """

        self.insurer = None
        """ Coverage issuer.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.outcome = None
        """ queued | complete | error | partial.
        Type `str`. """

        self.patient = None
        """ Intended recipient of products and services.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.preAuthRef = None
        """ Preauthorization reference.
        Type `str`. """

        self.purpose = None
        """ auth-requirements | benefits | discovery | validation.
        List of `str` items. """

        self.request = None
        """ Eligibility request reference.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.requestor = None
        """ Party responsible for the request.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.servicedDate = None
        """ Estimated date or dates of service.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.servicedPeriod = None
        """ Estimated date or dates of service.
        Type `Period` (represented as `dict` in JSON). """

        self.status = None
        """ active | cancelled | draft | entered-in-error.
        Type `str`. """

        super(CoverageEligibilityResponse, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CoverageEligibilityResponse, self).elementProperties()
        js.extend([
            ("created", "created", fhirdate.FHIRDate, "dateTime", False, None, True),
            ("disposition", "disposition", str, "string", False, None, False),
            ("error", "error", CoverageEligibilityResponseError, "CoverageEligibilityResponseError", True, None, False),
            ("form", "form", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", True, None, False),
            ("insurance", "insurance", CoverageEligibilityResponseInsurance, "CoverageEligibilityResponseInsurance", True, None, False),
            ("insurer", "insurer", fhirreference.FHIRReference, "Reference", False, None, True),
            ("outcome", "outcome", str, "code", False, None, True),
            ("patient", "patient", fhirreference.FHIRReference, "Reference", False, None, True),
            ("preAuthRef", "preAuthRef", str, "string", False, None, False),
            ("purpose", "purpose", str, "code", True, None, True),
            ("request", "request", fhirreference.FHIRReference, "Reference", False, None, True),
            ("requestor", "requestor", fhirreference.FHIRReference, "Reference", False, None, False),
            ("servicedDate", "servicedDate", fhirdate.FHIRDate, "date", False, "serviced", False),
            ("servicedPeriod", "servicedPeriod", period.Period, "Period", False, "serviced", False),
            ("status", "status", str, "code", False, None, True),
        ])
        return js


from . import backboneelement

class CoverageEligibilityResponseError(backboneelement.BackboneElement):
    """ Processing errors.

    Errors encountered during the processing of the request.
    """

    resource_type = "CoverageEligibilityResponseError"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.code = None
        """ Error code detailing processing issues.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(CoverageEligibilityResponseError, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CoverageEligibilityResponseError, self).elementProperties()
        js.extend([
            ("code", "code", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
        ])
        return js


class CoverageEligibilityResponseInsurance(backboneelement.BackboneElement):
    """ Patient insurance information.

    Financial instruments for reimbursement for the health care products and
    services.
    """

    resource_type = "CoverageEligibilityResponseInsurance"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.benefitPeriod = None
        """ When the benefits are applicable.
        Type `Period` (represented as `dict` in JSON). """

        self.coverage = None
        """ Insurance information.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.inforce = None
        """ Coverage inforce indicator.
        Type `bool`. """

        self.item = None
        """ Benefits and authorization details.
        List of `CoverageEligibilityResponseInsuranceItem` items (represented as `dict` in JSON). """

        super(CoverageEligibilityResponseInsurance, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CoverageEligibilityResponseInsurance, self).elementProperties()
        js.extend([
            ("benefitPeriod", "benefitPeriod", period.Period, "Period", False, None, False),
            ("coverage", "coverage", fhirreference.FHIRReference, "Reference", False, None, True),
            ("inforce", "inforce", bool, "boolean", False, None, False),
            ("item", "item", CoverageEligibilityResponseInsuranceItem, "CoverageEligibilityResponseInsuranceItem", True, None, False),
        ])
        return js


class CoverageEligibilityResponseInsuranceItem(backboneelement.BackboneElement):
    """ Benefits and authorization details.

    Benefits and optionally current balances, and authorization details by
    category or service.
    """

    resource_type = "CoverageEligibilityResponseInsuranceItem"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.authorizationRequired = None
        """ Authorization required flag.
        Type `bool`. """

        self.authorizationSupporting = None
        """ Type of required supporting materials.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.authorizationUrl = None
        """ Preauthorization requirements endpoint.
        Type `str`. """

        self.benefit = None
        """ Benefit Summary.
        List of `CoverageEligibilityResponseInsuranceItemBenefit` items (represented as `dict` in JSON). """

        self.category = None
        """ Benefit classification.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.description = None
        """ Description of the benefit or services covered.
        Type `str`. """

        self.excluded = None
        """ Excluded from the plan.
        Type `bool`. """

        self.modifier = None
        """ Product or service billing modifiers.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.name = None
        """ Short name for the benefit.
        Type `str`. """

        self.network = None
        """ In or out of network.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.productOrService = None
        """ Billing, service, product, or drug code.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.provider = None
        """ Performing practitioner.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.term = None
        """ Annual or lifetime.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.unit = None
        """ Individual or family.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(CoverageEligibilityResponseInsuranceItem, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CoverageEligibilityResponseInsuranceItem, self).elementProperties()
        js.extend([
            ("authorizationRequired", "authorizationRequired", bool, "boolean", False, None, False),
            ("authorizationSupporting", "authorizationSupporting", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("authorizationUrl", "authorizationUrl", str, "uri", False, None, False),
            ("benefit", "benefit", CoverageEligibilityResponseInsuranceItemBenefit, "CoverageEligibilityResponseInsuranceItemBenefit", True, None, False),
            ("category", "category", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("description", "description", str, "string", False, None, False),
            ("excluded", "excluded", bool, "boolean", False, None, False),
            ("modifier", "modifier", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("name", "name", str, "string", False, None, False),
            ("network", "network", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("productOrService", "productOrService", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("provider", "provider", fhirreference.FHIRReference, "Reference", False, None, False),
            ("term", "term", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("unit", "unit", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
        ])
        return js


class CoverageEligibilityResponseInsuranceItemBenefit(backboneelement.BackboneElement):
    """ Benefit Summary.

    Benefits used to date.
    """

    resource_type = "CoverageEligibilityResponseInsuranceItemBenefit"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.allowedMoney = None
        """ Benefits allowed.
        Type `Money` (represented as `dict` in JSON). """

        self.allowedString = None
        """ Benefits allowed.
        Type `str`. """

        self.allowedUnsignedInt = None
        """ Benefits allowed.
        Type `int`. """

        self.type = None
        """ Benefit classification.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.usedMoney = None
        """ Benefits used.
        Type `Money` (represented as `dict` in JSON). """

        self.usedString = None
        """ Benefits used.
        Type `str`. """

        self.usedUnsignedInt = None
        """ Benefits used.
        Type `int`. """

        super(CoverageEligibilityResponseInsuranceItemBenefit, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CoverageEligibilityResponseInsuranceItemBenefit, self).elementProperties()
        js.extend([
            ("allowedMoney", "allowedMoney", money.Money, "Money", False, "allowed", False),
            ("allowedString", "allowedString", str, "string", False, "allowed", False),
            ("allowedUnsignedInt", "allowedUnsignedInt", int, "unsignedInt", False, "allowed", False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
            ("usedMoney", "usedMoney", money.Money, "Money", False, "used", False),
            ("usedString", "usedString", str, "string", False, "used", False),
            ("usedUnsignedInt", "usedUnsignedInt", int, "unsignedInt", False, "used", False),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import money
except ImportError:
    money = sys.modules[__package__ + '.money']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/Signature) on 2019-05-13.
#  2019, SMART Health IT.


from . import element

class Signature(element.Element):
    """ A Signature - XML DigSig, JWS, Graphical image of signature, etc..

    A signature along with supporting context. The signature may be a digital
    signature that is cryptographic in nature, or some other signature
    acceptable to the domain. This other signature may be as simple as a
    graphical image representing a hand-written signature, or a signature
    ceremony Different signature approaches have different utilities.
    """

    resource_type = "Signature"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.data = None
        """ The actual signature content (XML DigSig. JWS, picture, etc.).
        Type `str`. """

        self.onBehalfOf = None
        """ The party represented.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.sigFormat = None
        """ The technical format of the signature.
        Type `str`. """

        self.targetFormat = None
        """ The technical format of the signed resources.
        Type `str`. """

        self.type = None
        """ Indication of the reason the entity signed the object(s).
        List of `Coding` items (represented as `dict` in JSON). """

        self.when = None
        """ When the signature was created.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.who = None
        """ Who signed.
        Type `FHIRReference` (represented as `dict` in JSON). """

        super(Signature, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(Signature, self).elementProperties()
        js.extend([
            ("data", "data", str, "base64Binary", False, None, False),
            ("onBehalfOf", "onBehalfOf", fhirreference.FHIRReference, "Reference", False, None, False),
            ("sigFormat", "sigFormat", str, "code", False, None, False),
            ("targetFormat", "targetFormat", str, "code", False, None, False),
            ("type", "type", coding.Coding, "Coding", True, None, True),
            ("when", "when", fhirdate.FHIRDate, "instant", False, None, True),
            ("who", "who", fhirreference.FHIRReference, "Reference", False, None, True),
        ])
        return js


import sys
try:
    from . import coding
except ImportError:
    coding = sys.modules[__package__ + '.coding']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b on 2019-05-13.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json

from .fixtures import force_bytes
from .. import medicinalproduct
from ..fhirdate import FHIRDate


@pytest.mark.usefixtures("base_settings")
class MedicinalProductTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("MedicinalProduct", js["resourceType"])
        return medicinalproduct.MedicinalProduct(js)
    
    def testMedicinalProduct1(self):
        inst = self.instantiate_from("medicinalproduct-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a MedicinalProduct instance")
        self.implMedicinalProduct1(inst)
        
        js = inst.as_json()
        self.assertEqual("MedicinalProduct", js["resourceType"])
        inst2 = medicinalproduct.MedicinalProduct(js)
        self.implMedicinalProduct1(inst2)
    
    def implMedicinalProduct1(self, inst):
        self.assertEqual(force_bytes(inst.id), force_bytes("example"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://ema.europa.eu/example/MPID"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("{mpid}"))
        self.assertEqual(force_bytes(inst.manufacturingBusinessOperation[0].authorisationReferenceNumber.system), force_bytes("http://ema.europa.eu/example/manufacturingAuthorisationReferenceNumber"))
        self.assertEqual(force_bytes(inst.manufacturingBusinessOperation[0].authorisationReferenceNumber.value), force_bytes("1324TZ"))
        self.assertEqual(inst.manufacturingBusinessOperation[0].effectiveDate.date, FHIRDate("2013-03-15").date)
        self.assertEqual(inst.manufacturingBusinessOperation[0].effectiveDate.as_json(), "2013-03-15")
        self.assertEqual(force_bytes(inst.manufacturingBusinessOperation[0].operationType.coding[0].code), force_bytes("Batchrelease"))
        self.assertEqual(force_bytes(inst.manufacturingBusinessOperation[0].operationType.coding[0].system), force_bytes("http://ema.europa.eu/example/manufacturingOperationType"))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.name[0].countryLanguage[0].country.coding[0].code), force_bytes("EU"))
        self.assertEqual(force_bytes(inst.name[0].countryLanguage[0].country.coding[0].system), force_bytes("http://ema.europa.eu/example/countryCode"))
        self.assertEqual(force_bytes(inst.name[0].countryLanguage[0].jurisdiction.coding[0].code), force_bytes("EU"))
        self.assertEqual(force_bytes(inst.name[0].countryLanguage[0].jurisdiction.coding[0].system), force_bytes("http://ema.europa.eu/example/jurisdictionCode"))
        self.assertEqual(force_bytes(inst.name[0].countryLanguage[0].language.coding[0].code), force_bytes("EN"))
        self.assertEqual(force_bytes(inst.name[0].countryLanguage[0].language.coding[0].system), force_bytes("http://ema.europa.eu/example/languageCode"))
        self.assertEqual(force_bytes(inst.name[0].namePart[0].part), force_bytes("Equilidem"))
        self.assertEqual(force_bytes(inst.name[0].namePart[0].type.code), force_bytes("INV"))
        self.assertEqual(force_bytes(inst.name[0].namePart[1].part), force_bytes("2.5 mg"))
        self.assertEqual(force_bytes(inst.name[0].namePart[1].type.code), force_bytes("STR"))
        self.assertEqual(force_bytes(inst.name[0].namePart[2].part), force_bytes("film-coated tablets"))
        self.assertEqual(force_bytes(inst.name[0].namePart[2].type.code), force_bytes("FRM"))
        self.assertEqual(force_bytes(inst.name[0].productName), force_bytes("Equilidem 2.5 mg film-coated tablets"))
        self.assertEqual(force_bytes(inst.productClassification[0].coding[0].code), force_bytes("WHOAnatomicalTherapeuticChemicalATCClassificationSystem|B01AF02"))
        self.assertEqual(force_bytes(inst.productClassification[0].coding[0].system), force_bytes("http://ema.europa.eu/example/WHOAnatomicalTherapeuticChemicalATCClassificationSystem"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))

